#include <avr/wdt.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define ATTR_INIT_SECTION(SectionIndex) __attribute__ ((used, naked, section (".init" #SectionIndex )))
#define ATTR_NO_INIT                    __attribute__ ((section (".noinit")))

#define MAGIC_BOOT_KEY            0xDC42ACCA
#if __AVR_ATmega16U2__
#   define BOOTLOADER_START_ADDRESS  0x0800 // (FLASH_SIZE_BYTES - BOOTLOADER_SEC_SIZE_BYTES)
#elif __AVR_ATmega32U2__   
#   define BOOTLOADER_START_ADDRESS  0x3800 // (FLASH_SIZE_BYTES - BOOTLOADER_SEC_SIZE_BYTES)
#else
#   error "Unknown device name " __AVR_DEVICE_NAME__
#endif

uint32_t boot_flag ATTR_NO_INIT;
void reset_check_bootflag() ATTR_INIT_SECTION(3);

void reset_check_bootflag() {
    // If the reset source was the bootloader and the key is correct, clear it and jump to the bootloader
    if ((MCUSR & (1 << WDRF)) && (boot_flag == MAGIC_BOOT_KEY)) {
        boot_flag = 0;
        ((void (*)(void))BOOTLOADER_START_ADDRESS)();
    }
}

void reset_dfumode() {
    // Disable all interrupts
    cli();
    
    // Set the bootloader key to the magic value and force a reset
    boot_flag = MAGIC_BOOT_KEY;
    wdt_enable(WDTO_250MS);
    for (;;);
}